#! /bin/sh

# NOTE:  Todo.sh requires the .todo configuration file to run.
# Place the .todo file in your home directory or use the -d option for a custom location.

# To adapt to System V vs. BSD 'echo'
if echo '\c' | grep -s c > /dev/null ; then
    # BSD
    NOCR1='-n'
    NOCR2=""
else
    # System V
    NOCR1=""
    NOCR2='\c'
fi

version() { sed -e 's/^        //' <<EndVersion
        TODO.TXT Migrator, Adapted from todo.sh
        Version L1.0.0
        Author:  Gina Trapani (ginatrapani@gmail.com)
        Modified: Lorance Stinson <LoranceStinson+todo@gmail.com>
        Release date: 2006-10-22
        Last updated: 2006-10-22
        License:  GPL, http://www.gnu.org/copyleft/gpl.html
        More information and mailing list at http://todotxt.com
EndVersion
    exit 1
}

usage()
{
    sed -e 's/^    //' <<EndUsage 
    Usage: `basename $0` [-hu] [-d todo_config]
      Adds unique ID's to todo.txt and done.txt.  Creates the history and ID
      files and the comments directory.  Adds the new variables to your .todo
      file.  No actions are performed without the corresponding option.

      Options:
        -c
            Create the new files id.txt and history.txt and the new directory
            comments that the new todo.sh requires.
        -d CONFIG_FILE
            Use a configuration file other than the default ~/.todo
        -i ID
            Starting ID to use when creating the ID file. Defaults to AA.
        -h
            Display this help message
        -n
            Renumber the todo.txt and done.txt files. This step must be performed
            ONCE for the new todo.sh to work. 
        -u
            Update the .todo configuration file.
        -V 
            Displays version, license and credits
EndUsage
    exit 1
}

# These two work like Number::RecordLocator used by hiveminder.com.
# Since 0,1,S and B tend be be confused for O, I, F and P respectively by look
# or sound the former is transformed into the latter. This is neatest part of
# the module.
# Convert a base 10 number into a base 32 code.
encode_id()
{
    echo "$1" | awk '{id="";while($1>0){id=substr("23456789ACDEFGHIJKLMNOPQRTUVWXYZ",($1%32)+1,1)id;$1=int($1/32)}print id}'
}

# Convert a base 32 code into a base 10 number.
decode_id()
{
    echo $1 | tr 'a-z' 'A-Z' | tr '01SB' 'OIFP' | \
    awk '{num=0;for(i=1;i<length($1)+1;i++){num=(num*32)+index("23456789ACDEFGHIJKLMNOPQRTUVWXYZ",substr($1,i,1))-1}print num}'
}

die()
{
    echo "$*"
    exit 1
}

# == PROCESS OPTIONS ==
# defaults
CFG_FILE=$HOME/.todo
CREATE=0
NUMBER=0
START_ID="AA"
UPDATE=0

while getopts ":chinVud:" OPTION ; do
    case $OPTION in
        c) CREATE=1 ;;
        d) CFG_FILE=$OPTARG ;;
        i) START_ID=$OPTARG ;;
        h) usage ;;
        n) NUMBER=1 ;;
        u) UPDATE=1 ;;
        V) version ;;
    esac
done
shift `expr $OPTIND - 1`

# === SANITY CHECKS (thanks Karl!) ===
[ -r "$CFG_FILE" ] || die "Fatal error:  Cannot read configuration file $CFG_FILE"

. "$CFG_FILE" > /dev/null 2>&1 || die "Could not read the configuration file $CFG_FILE."

[ ! -z "$TODO_DIR" -a -d "$TODO_DIR" ] || die "Could not find the todo directory."

# Update the config file if requested.
if [ $UPDATE -eq 1 ] ; then
    # See if it looks like this has been done before.
    if grep "^HISTORY=" $CFG_FILE > /dev/null 2>&1 ; then
        echo "It looks like the new variables have already been added."
        echo "Please check `basename $CFG_FILE` to make sure."
        echo ""
    elif [ -f $CFG_FILE.org ] ; then
        echo "The backup file `basename $CFG_FILE`.org already exists."
        echo "This probably means it has already been updated."
        echo "`basename $CFG_FILE` has not been modified."
    else
        # Backup the config file.
        cp $CFG_FILE $CFG_FILE.org
        echo "A backup of '`basename $CFG_FILE`' has been made as '`basename $CFG_FILE`.org'."

        # Add the variables.
        awk -v date="`date +"%Y-%m-%d %T"`" \
            'BEGIN {
                printf "# Updated for changes by Lorance Stinson: %s\n", date
            }
            { print $0;}
            /^TMP_FILE=/ {
                print ""
                print "# New files."
                print "ID_FILE=\"$TODO_DIR/id.txt\""
                print "HIST_FILE=\"$TODO_DIR/history.txt\""
                print "COMMENT_DIR=\"$TODO_DIR/comments\""
            }
            /^NONE=/ { print "ESC=`echo \"\" | tr \"\\n\" \"\\033\"`"; }
            /^PRI_X/ { print "DONE=$RED       # color for completed todos"; }
            END { # Write out the new variables to the end of the file.
                print ""
                print "# Keep a history of all changes."
                print "# If this is set to a true value a history of all activities that alters the"
                print "# todo.txt file is kept in the file specified by HIST_FILE above."
                print "# To disable the history function simply comment out this line."
                print "HISTORY=1"
            }' $CFG_FILE.org | \
            sed -e 's/.\\\\033\(.*\)./"\$ESC\1"/' > $CFG_FILE

        # Let the user know.
        echo "The new variables have been added."
        echo ""
    fi
fi

# Create the new files if they do not already exist if requested..
if [ $CREATE -eq 1 ] ; then
    # History file.
    [ -z "$HIST_FILE" ] && HIST_FILE="$TODO_DIR/history.txt"
    if [ ! -f "$HIST_FILE" ] ; then
        cp /dev/null "$HIST_FILE"
        echo "Created history file: $HIST_FILE"
        echo ""
    fi

    # ID file.
    [ -z "$ID_FILE" ] && ID_FILE="$TODO_DIR/id.txt"
    if [ ! -f "$ID_FILE" ] ; then
        echo "`decode_id "$START_ID"`" > "$ID_FILE"
        echo "Created ID file: $ID_FILE"
        echo ""
    fi

    # Comments directory.
    [ -z "$COMMENT_DIR" ] && COMMENT_DIR="$TODO_DIR/comments"
    if [ ! -d "$COMMENT_DIR" ] ; then
        mkdir $COMMENT_DIR
        echo "Created comments directory: $COMMENT_DIR"
        echo ""
    fi
fi

# Add the ID to todo.txt and done.txt if requested.
if [ $NUMBER -eq 1 ] ; then
    # Don't add the ID again if the backup files exist.
    [ -f $TODO_FILE.org -a -f $DONE_FILE.org ] && \
        echo "The backup files `basename $TODO_FILE`.org and `basename $DONE_FILE`.org already exist." && \
        echo "This probably means they have already been updated." && \
        echo "`basename $TODO_FILE` and `basename $DONE_FILE` have not been modified." && \
        exit 1

    # Check to see if it looks like todo.txt already has the new ID's.
    TODO_LINES=`wc -l "$TODO_FILE" | sed 's/^[[:space:]]*\([0-9]*\).*/\1/'`
    CHECK_LINES=`grep '^[A-Z0-9][A-Z0-9]*: ' "$TODO_FILE" | wc -l | sed 's/^[[:space:]]*\([0-9]*\).*/\1/'`
    if [ $TODO_LINES -eq $CHECK_LINES ] ; then
        echo "It looks like `basename $TODO_FILE` has already been updated."
        echo "Every line has what looks like the new ID."
        echo "The new ID matches the following RE: '^[A-Z0-9][A-Z0-9]*: '."
        exit 1
    fi

    # Backup the files before changing them.
    cp $TODO_FILE $TODO_FILE.org
    cp $DONE_FILE $DONE_FILE.org

    # Make sure the backups were created.
    [ -f $TODO_FILE.org -a -f $DONE_FILE.org ] || die "Failed to create backup files."

    # Let the user know what is going on.
    echo "Adding unique ID's to todos in `basename $TODO_FILE` and `basename $DONE_FILE`."
    echo "Backups have been made as `basename $TODO_FILE`.org and `basename $DONE_FILE`.org."
    echo "Do not delete them before making sure the migration process has worked."

    # Add IDs to each file.
    # First todo.txt and then done.txt.
    TASKID=`cat $ID_FILE`
    while read input ; do
        TASKID=`expr $TASKID + 1`
        echo "`encode_id "$TASKID"`: $input"
    done < $TODO_FILE.org > $TODO_FILE

    while read input ; do
        TASKID=`expr $TASKID + 1`
        echo "`encode_id "$TASKID"`: $input"
    done < $DONE_FILE.org > $DONE_FILE

    # Save the ID.
    echo "$TASKID" > $ID_FILE
fi
